import 'dart:async';

import 'package:businesslibrary/data/delivery_note.dart';
import 'package:businesslibrary/data/invoice.dart';
import 'package:businesslibrary/data/invoice_bid.dart';
import 'package:businesslibrary/data/offer.dart';
import 'package:businesslibrary/data/purchase_order.dart';
import 'package:cloud_firestore/cloud_firestore.dart';

class ListAPI {
  static final Firestore _firestore = Firestore.instance;

  static Future<List<InvoiceBid>> getInvoiceBidsByOffer(String offer) async {
    List<InvoiceBid> list = List();
    var qs = await _firestore
        .collection('invoiceBids')
        .where('offer', isEqualTo: offer)
        .orderBy('date', descending: true)
        .getDocuments()
        .catchError((e) {
      print('ListAPI.getOfferInvoiceBids $e');
      return null;
    });

    print('ListAPI.getOfferInvoiceBids found: ${qs.documents.length} ');

    qs.documents.forEach((doc) async {
      list.add(new InvoiceBid.fromJson(doc.data));
    });

    return list;
  }

  static Future<List<InvoiceBid>> getInvoiceBidsByInvestor(
      String participantId) async {
    List<InvoiceBid> list = List();
    var qs = await _firestore
        .collection('invoiceBids')
        .where('participantId', isEqualTo: participantId)
        .orderBy('date', descending: true)
        .getDocuments()
        .catchError((e) {
      print('ListAPI.getInvestorInvoiceBids $e');
      return null;
    });

    print('ListAPI.getInvestorInvoiceBids found: ${qs.documents.length} ');

    qs.documents.forEach((doc) async {
      list.add(new InvoiceBid.fromJson(doc.data));
    });

    return list;
  }

  static Future<List<Offer>> getOffersByPeriod(
      DateTime startTime, DateTime endTime) async {
    List<Offer> list = List();
    var qs = await _firestore
        .collection('invoiceOffers')
        .where('date', isGreaterThanOrEqualTo: startTime.toIso8601String())
        .where('date', isLessThanOrEqualTo: endTime.toIso8601String())
        .orderBy('date', descending: true)
        .getDocuments()
        .catchError((e) {
      print('ListAPI.getSupplierOffers $e');
      return null;
    });

    print('ListAPI.getSupplierOffers found: ${qs.documents.length} ');

    qs.documents.forEach((doc) async {
      list.add(new Offer.fromJson(doc.data));
    });

    return list;
  }

  static Future<List<Offer>> getOffersBySector(
      String privateSectorType, DateTime startTime, DateTime endTime) async {
    List<Offer> list = List();
    var qs = await _firestore
        .collection('invoiceOffers')
        .where('date', isGreaterThanOrEqualTo: startTime.toIso8601String())
        .where('date', isLessThanOrEqualTo: endTime.toIso8601String())
        .where('privateSectorType', isEqualTo: privateSectorType)
        .orderBy('date', descending: true)
        .getDocuments()
        .catchError((e) {
      print('ListAPI.getSupplierOffers $e');
      return null;
    });

    print('ListAPI.getSupplierOffers found: ${qs.documents.length} ');

    qs.documents.forEach((doc) async {
      list.add(new Offer.fromJson(doc.data));
    });

    return list;
  }

  static Future<List<Offer>> getOffersBySupplier(String supplierId) async {
    List<Offer> list = List();
    var qs = await _firestore
        .collection('invoiceOffers')
        .where('participantId', isEqualTo: supplierId)
        .orderBy('date', descending: true)
        .getDocuments()
        .catchError((e) {
      print('ListAPI.getSupplierOffers $e');
      return null;
    });

    print('ListAPI.getSupplierOffers found: ${qs.documents.length} ');

    qs.documents.forEach((doc) async {
      list.add(new Offer.fromJson(doc.data));
    });

    return list;
  }

  static Future<List<PurchaseOrder>> getPurchaseOrders(
      String participantId, String collection) async {
    List<PurchaseOrder> list = List();
    var qs = await _firestore
        .collection(collection)
        .where('participantId', isEqualTo: participantId)
        .getDocuments()
        .catchError((e) {
      print('ListAPI.getPurchaseOrders $e');
      return null;
    });

    print('ListAPI.getPurchaseOrders main doc found: ${qs.documents.length} ');

    qs.documents.forEach((doc) async {
      var qs0 = await doc.reference.collection('purchaseOrders').getDocuments();

      qs0.documents.forEach((doc) {
        list.add(new PurchaseOrder.fromJson(doc.data));
      });
    });

    print(
        'ListAPI.getPurchaseOrders ############## docs found: ${list.length}');
    return list;
  }

  static Future<List<Invoice>> getInvoices(
      String participantId, String collection) async {
    List<Invoice> list = List();
    var qs = await _firestore
        .collection(collection)
        .where('participantId', isEqualTo: participantId)
        .getDocuments()
        .catchError((e) {
      print('ListAPI.getInvoices $e');
      return null;
    });

    print('ListAPI.getInvoices main doc found: ${qs.documents.length} ');
    qs.documents.forEach((doc) async {
      var qs0 = await doc.reference.collection('invoices').getDocuments();

      qs0.documents.forEach((doc) {
        list.add(new Invoice.fromJson(doc.data));
      });
    });

    print('ListAPI.getInvoices ################## found: ${list.length}');
    return list;
  }

  static Future<List<DeliveryNote>> getDeliveryNotes(
      String participantId, String collection) async {
    List<DeliveryNote> list = List();
    var qs = await _firestore
        .collection(collection)
        .where('participantId', isEqualTo: participantId)
        .getDocuments()
        .catchError((e) {
      print('ListAPI.getDeliveryNotes $e');
      return null;
    });

    print('ListAPI.getDeliveryNotes main doc found: ${qs.documents.length} ');
    qs.documents.forEach((doc) async {
      var qs0 = await doc.reference
          .collection('invoices')
          .orderBy('date', descending: true)
          .getDocuments();

      qs0.documents.forEach((doc) {
        list.add(new DeliveryNote.fromJson(doc.data));
      });
    });

    print('ListAPI.getDeliveryNotes found: ${list.length}');
    return list;
  }
}
